//---------------------------------------------------------------------------------
// File Name: DllTestC.c
// Description:
//
// Copyright (C) 2004 - 2005 True Axis Pty Ltd, Australia. 
// All Rights Reserved.
//
//---------------------------------------------------------------------------------

#include "../Physics/Physics.h"
#include "../Physics/DynamicObject.h"
#include "../Physics/StaticObject.h"
#include "../Physics/CollisionObjectAABBMesh.h"

const float k_fWorldExtent = 1000.0f;
const float k_fGroundExtent = 10.0f;

//---------------------------------------------------------------------------------
// A simple physics simulation
// Y is up.
//---------------------------------------------------------------------------------
void dllTestC()
{
    TaAABB aabb;
	TaVec3 v3Gravity = { 0.0f, -9.81f, 0.0f };
	TaVec3 v3Position;
	TaPhysics* pPhysics = 0;
	TaDynamicObject* pDynamicObject = 0;
	TaStaticObject* pStaticObject = 0;
	TaCollisionObjectAABBMesh* pStaticCollisionObject = 0;


    //---------------------------------------------------------------------------------
    // Initialise physics.
    //---------------------------------------------------------------------------------
	aabb.v3Center.x = 0.0f;
	aabb.v3Center.y = 0.0f;
	aabb.v3Center.z = 0.0f;
	aabb.v3Extent.x = k_fWorldExtent;
	aabb.v3Extent.y = k_fWorldExtent;
	aabb.v3Extent.z = k_fWorldExtent;


    TaPhysics_CreateInstance2(
        &aabb,                                  // World bounding box.
        &v3Gravity,                             // Gravity.
        TA_PHYSICS_FLAG_XZ_COLLISION_GRID,      // Flags. 
		10.0f,                                  // Min sub-division.
		0x40000);                               // Max memory.

    // Make a reference to the physics instance.
    // Would normaly just use TaPhysics::GetInstance() directly
    // but the automatic documentation generator doesn't recognise it.
    pPhysics = TaPhysics_GetInstance();


    //---------------------------------------------------------------------------------
    // Add a dynamic object.
    //---------------------------------------------------------------------------------
    pDynamicObject = TaDynamicObject_CreateNew();
	// Make a 1 by 1 by 1 box.
	aabb.v3Center.x = 0.0f;
	aabb.v3Center.y = 0.0f;
	aabb.v3Center.z = 0.0f;
	aabb.v3Extent.x = 0.5f;
	aabb.v3Extent.y = 0.5f;
	aabb.v3Extent.z = 0.5f;
    TaDynamicObject_InitialiseAsABox(pDynamicObject, &aabb);

    // Place the dynamic object
	v3Position.x = 0.0f;
	v3Position.y = 2.0f, 
	v3Position.z = 0.0f;
	TaDynamicObject_SetPosition(pDynamicObject, &v3Position);

    // Add the dynamic object to the simulation
    TaPhysics_AddDynamicObject(pPhysics, pDynamicObject, TaTrue);


    //---------------------------------------------------------------------------------
    // Add a static object for the ground.
    //---------------------------------------------------------------------------------
    pStaticObject = TaStaticObject_CreateNew();

    // Create a collision object to add to the static object.
    pStaticCollisionObject = TaCollisionObjectAABBMesh_CreateNew();
    TaCollisionObjectAABBMesh_Initialise(
		pStaticCollisionObject,
        4,              // Num vertices.
        1,              // Num polygons.
        4);             // Num polygon indices.

	v3Position.x =  k_fGroundExtent;   v3Position.y = 0.0f;   v3Position.z =  k_fGroundExtent;
	TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, &v3Position);

	v3Position.x = -k_fGroundExtent;   v3Position.y = 0.0f;   v3Position.z =  k_fGroundExtent;
    TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, &v3Position);

	v3Position.x = -k_fGroundExtent;   v3Position.y = 0.0f;   v3Position.z = -k_fGroundExtent;
    TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, &v3Position);

	v3Position.x =  k_fGroundExtent;   v3Position.y = 0.0f;   v3Position.z = -k_fGroundExtent;
    TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, &v3Position);


	{
		int pnPolygonIndexList[4] = { 0, 1, 2, 3 };
		TaCollisionObjectAABBMesh_AddPolygon(pStaticCollisionObject, 4, pnPolygonIndexList, 0);
	}
	TaCollisionObjectAABBMesh_FinishedAddingGeometry(pStaticCollisionObject);

    // Initialise the static object with the collision object.
    TaStaticObject_Initialise(
		pStaticObject, 
		TaCollisionObjectAABBMesh_GetTaCollisionObjectComplex(pStaticCollisionObject));
    TaCollisionObjectAABBMesh_Release(pStaticCollisionObject); // We no long need the reference.
    pStaticCollisionObject = 0;

    // Add the static object to the simulation.
    TaPhysics_AddStaticObject(pPhysics, pStaticObject);
    TaStaticObject_Release(pStaticObject); // We no long need the reference.
    pStaticObject = 0;


    //---------------------------------------------------------------------------------
    // Other initialisation.
    //---------------------------------------------------------------------------------
    //.
    //.
    //.

    //---------------------------------------------------------------------------------
    // Main loop.
    //---------------------------------------------------------------------------------
    for (;;) 
    {
        float fDt = 1.0f / 60.0f; // Time of current frame

        // Caculate value for fDt
        //.
        //.
        //.

        // Update the simulation
        TaPhysics_Update(pPhysics, fDt);

        // Do game logic and render the scene
        //.
        //.
        //.

        // Use pDynamicObject->GetFrame() to render
        // the dynamic object.
        //.
        //.
        //.
    }


    //---------------------------------------------------------------------------------
    // Finalise physics.
    //---------------------------------------------------------------------------------
    TaPhysics_DestroyInstance(); 

    TaDynamicObject_Release(pDynamicObject);
    pDynamicObject = 0;


    //---------------------------------------------------------------------------------
    // Other finalisation.
    //---------------------------------------------------------------------------------
    //.
    //.
    //.
}

