program main;

uses TaPhysics, TaDynamicObject, TaStaticObject, TaCollisionObjectAABBMesh, TaTypes, TaCollision;

      
const
  k_fWorldExtent: Single = 1000.0;   
  k_fGroundExtent: Single = 10.0;

var
  s: String;
  fDt : Single;
  pPhysics: PTaPhysics;
  v3Gravity: TTaVec3;
  aabb: TTaAABB;
  v3Position: TTaVec3;
  pDynamicObject: PTaDynamicObject;
  pStaticObject: PTaStaticObject;
  pStaticCollisionObject : PTaCollisionObjectAABBMesh;
  pnPolygonIndexList : array [0..3] of Integer;
  collision :TTaCollision;
  v3Start: TTaVec3;
  v3End: TTaVec3;

procedure ProcessCollisions(var collision : TTaCollision); stdcall; 
  var v3Position : TTaVec3;
begin
  v3Position := TaCollision_GetPosition(@collision)[0];

 WriteLn('Collision Position ', v3Position.x, ' ', v3Position.y, ' ', v3Position.z);

end;

begin
    { Initialise physics }

    v3Gravity.x := 0;
    v3Gravity.y := -9.81;
    v3Gravity.z := 0;

    aabb.v3Center.x := 0.0;
	aabb.v3Center.y := 0.0;
	aabb.v3Center.z := 0.0;
	aabb.v3Extent.x := k_fWorldExtent;
	aabb.v3Extent.y := k_fWorldExtent;
	aabb.v3Extent.z := k_fWorldExtent;

    TaPhysics_CreateInstance2(
        aabb,
        v3Gravity,
        TA_PHYSICS_FLAG_XZ_COLLISION_GRID,
        10.0,
        $40000);
        
    pPhysics := TaPhysics_GetInstance();  
                   
    { Check what collisions will occure during update. }
    TaPhysics_SetPostProcessCollisionCallBack(pPhysics, @ProcessCollisions);

    { Add a dynamic object. }
    pDynamicObject := TaDynamicObject_CreateNew();
	{ Make a 1 by 1 by 1 box.}
	aabb.v3Center.x := 0.0;
	aabb.v3Center.y := 0.0;
	aabb.v3Center.z := 0.0;
	aabb.v3Extent.x := 0.5;
	aabb.v3Extent.y := 0.5;
	aabb.v3Extent.z := 0.5;
    TaDynamicObject_InitialiseAsABox(pDynamicObject, aabb);

    { Place the dynamic object }
	v3Position.x := 0.0;
	v3Position.y := 2.0;
	v3Position.z := 0.0;
	TaDynamicObject_SetPosition(pDynamicObject, v3Position);

    { Add the dynamic object to the simulation  }
    TaPhysics_AddDynamicObject(pPhysics, pDynamicObject, TaTrue);



      //---------------------------------------------------------------------------------
    // Add a static object for the ground.
    //---------------------------------------------------------------------------------
    pStaticObject := TaStaticObject_CreateNew();

    // Create a collision object to add to the static object.
    pStaticCollisionObject := TaCollisionObjectAABBMesh_CreateNew();
    TaCollisionObjectAABBMesh_Initialise(
		pStaticCollisionObject,
        4,              // Num vertices.
        1,              // Num polygons.
        4);             // Num polygon indices.

	v3Position.x :=  k_fGroundExtent;   v3Position.y := 0.0;   v3Position.z :=  k_fGroundExtent;
	TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, v3Position);

	v3Position.x := -k_fGroundExtent;   v3Position.y := 0.0;   v3Position.z :=  k_fGroundExtent;
    TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, v3Position);

	v3Position.x := -k_fGroundExtent;   v3Position.y := 0.0;   v3Position.z := -k_fGroundExtent;
    TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, v3Position);

	v3Position.x :=  k_fGroundExtent;   v3Position.y := 0.0;   v3Position.z := -k_fGroundExtent;
    TaCollisionObjectAABBMesh_AddVertex(pStaticCollisionObject, v3Position);


    pnPolygonIndexList[0] := 0;
    pnPolygonIndexList[1] := 1;
    pnPolygonIndexList[2] := 2;
    pnPolygonIndexList[3] := 3;
	TaCollisionObjectAABBMesh_AddPolygon(pStaticCollisionObject, 4, @pnPolygonIndexList[0], 0);
	TaCollisionObjectAABBMesh_FinishedAddingGeometry(pStaticCollisionObject);

    { Initialise the static object with the collision object. }
    TaStaticObject_Initialise(
		pStaticObject, 
		TaCollisionObjectAABBMesh_GetTaCollisionObjectComplex(pStaticCollisionObject));
    TaCollisionObjectAABBMesh_Release(pStaticCollisionObject); // We no long need the reference.
    pStaticCollisionObject := 0;

    { Add the static object to the simulation. }
    TaPhysics_AddStaticObject(pPhysics, pStaticObject);
    TaStaticObject_Release(pStaticObject); // We no long need the reference.
    pStaticObject := 0;


    { Other initialisation.
    .
    .
    .
    }


    { TaPhysics_TestLineForCollision2 exampe. }
    v3Start.x := 0;
    v3Start.y := 20;
    v3Start.z := 0;
    v3End.x := 0;
    v3End.y := -20;
    v3End.z := 0;
    TaPhysics_TestLineForCollision2(
        pPhysics,
        v3Start,
        v3End,
        @collision,
        TA_PHYSICS_FLAG_ALL_OBJECTS);
    if TaCollision_CollisionOccurred(@collision) then
      begin
        v3Position := TaCollision_GetPosition(@collision)[0];
        WriteLn('TaPhysics_TestLineForCollision2 '' ', v3Position.x, ' ', v3Position.y, ' ', v3Position.z);
      end;


    { Main loop.  }
    while True do
      begin
        fDt := 1.0 / 60.0; { Time of current frame }

        { Caculate value for fDt
        .
        .
        .
        }



        { Update the simulation  }
        TaPhysics_Update(pPhysics, fDt);  
        v3Position := TaDynamicObject_GetPosition(pDynamicObject)[0];
        WriteLn('Position ', v3Position.x, ' ', v3Position.y, ' ', v3Position.z);
        ReadLn (s);

        { Do game logic and render the scene
        .
        .
        .
        }

        { Use pDynamicObject->GetFrame() to render
          the dynamic object.
        .
        .
        .
        }

      end;


    { Finalise physics. }
    TaPhysics_DestroyInstance();

    TaDynamicObject_Release(pDynamicObject);
    pDynamicObject := 0;

end.

